###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.9 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS
import OFS.Folder
import Products
import base_item
import mail_gateway
import mailbox
import thread
import cStringIO
import re
import message_container
import news_group_folder
import message_filter_folder

from Products import RFC822Message

gum_add_redirect = HTMLFile('gum_add_redirect', globals())
manage_add_gum_form = HTMLFile('gum_add', globals())


def manage_add_gum(self, id, local_username, email_address, title,
				   REQUEST=None, RESPONSE=None):
    "Add an instance of the gum class"

    self._setObject(id, gum(id, local_username, email_address, title))

    if REQUEST is not None:

		RESPONSE.redirect(self.absolute_url() + '/' + id)


class gum(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	counter = 0

	meta_type = 'GUM'
	icon = 'misc_/GUM/Folder_icon.gif'
	#default_catalog = 'Catalog'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'local_username', 'type':'string'},
		{'id':'title', 'type':'string'},
		{'id':'email_address', 'type':'string'},
		)

	meta_types = (
		{'name':'MailGateway',
		 'action':'manage_add_mail_gateway_form'},
		{'name':'MessageContainer',
		 'action':'manage_add_message_container_form'},
		{'name':'NewsGroupFolder',
		 'action':'manage_add_news_group_folder_form'},
		{'name':'MessageFilterFolder',
		 'action':'manage_add_message_filter_folder_form'}
		)

	manage_add_mail_gateway_form = mail_gateway.manage_add_mail_gateway_form
	manage_add_mail_gateway = mail_gateway.manage_add_mail_gateway
	mail_gateway_add_redirect = mail_gateway.add_mail_gateway_redirect

	manage_add_message_container_form = message_container.manage_add_message_container_form
	manage_add_message_container = message_container.manage_add_message_container
	message_container_add_redirect = message_container.message_container_add_redirect

	manage_add_news_group_folder_form = news_group_folder.manage_add_news_group_folder_form
	manage_add_news_group_folder = news_group_folder.manage_add_news_group_folder
	news_group_folder_add_redirect = news_group_folder.news_group_folder_add_redirect

	manage_add_message_filter_folder_form = message_filter_folder.manage_add_message_filter_folder_form
	manage_add_message_filter_folder = message_filter_folder.manage_add_message_filter_folder
	message_filter_folder_add_redirect = message_filter_folder.message_filter_folder_add_redirect	

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add MailGateway',
		 ('manage_add_mail_gateway', 'manage_add_mail_gateway_form')),
		('Add MessageContainer',
		 ('manage_add_message_container', 'manage_add_message_container_form')),
		('View GUMs',
		 ('',)),
		)

	index_html = HTMLFile('gum_index', globals())

	base_edit = HTMLFile('gum_edit', globals())

	preferences = HTMLFile('gum_preferences', globals())

	product_name = 'GUM'


	def __init__(self, id, local_username, email_address, title='',
				 primary_folder='Inbox'):
		"Initialize the class instance"

		# Add Inbox and Outgoing

		message_container.manage_add_message_container(self, 'Inbox', 'Inbox')
		message_container.manage_add_message_container(self, 'Outgoing',
													   'Outgoing')		
		message_container.manage_add_message_container(self, 'Temporary',
													   'Temporary')
		message_container.manage_add_message_container(self, 'Drafts', 'Drafts')

		message_filter_folder.manage_add_message_filter_folder(
			self, 'MessageFilters', 'MessageFilters')

		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_header',
									 'standard_html_header',
									 open('%s/Products/%s/standard_html_header.dtml' % \
										  (SOFTWARE_HOME, self.product_name)).read())
										  
		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_footer',
									 'standard_html_footer',
									 open('%s/Products/%s/standard_html_footer.dtml' % \
										  (SOFTWARE_HOME, self.product_name)).read())

		OFS.DTMLMethod.addDTMLMethod(self, 'standard_css', 'standard_css',
									 open('%s/Products/%s/standard_css.dtml' % \
										  (SOFTWARE_HOME, self.product_name)).read())


		self.id = id
		self.title = title

		self.local_username = local_username
		self.email_address = email_address

		self.primary_folder = primary_folder

		self.listing_size = 15

		try:
		
			self.index_object()

		except AttributeError:

			# No ZCatalog instance available.

			pass


	def get_unique_id(self):
		"""Returns a unique identifier."""

		try:

			return self.getParentNode().get_id()

		except AttributeError:

			self.counter = self.counter + 1

			return str(self.counter)


	def get_id(self):
		"""Hack, do not use this method."""

		# FIXME

		print 'get_id was used..'
		return self.get_unique_id()


	def get_temporary_folder(self):
		"""Returns the temporary folder."""

		# FIXME

		return self['Temporary']

	def get_draft_folder(self):
		"""Returns the draft folder."""

		# FIXME

		return self['Drafts']


	def get_gum(self):
		"""Returns the GUM instance."""

		return self


	def get_mail_gateway(self):
		"""Returns default mail gateway."""

		try:

			return self[self.default_mail_gateway]

		except AttributeError:

			return self.objectValues('MailGateway')[0]


	def get_news_host(self):
		"""Returns newshost."""

		# FIXME -- this is not (does not feel like) the right
		# way to do it..

		news_hosts = []

		reference = self

		while 1:

			news_hosts = reference.objectValues('NewsHost')

			if news_hosts:

				break

			reference = reference.getParentNode()

		if not news_hosts:

			raise 'NoNewsHosts', 'No news hosts are configured on this server'

		return news_hosts[0]


	def get_gum_version(self):
		"""Returns a version number."""

		return open(SOFTWARE_HOME + '/Products/' + self.product_name + '/version.txt').read()


	def get_gum_website(self):
		"""Returns the GUM website."""

		return open(SOFTWARE_HOME + '/Products/' + self.product_name + '/website.txt').read()


	def retrieve_messages(self, REQUEST=None, RESPONSE=None, run_as_thread=0):
		"""Wrapper so we can choose whether to retrieve messages in thread
		mode."""

		if run_as_thread:

			thread.start_new_thread(self._retrieve_messages, (self,),
						{'REQUEST':REQUEST, 'RESPONSE':RESPONSE})
			return 'started thread'

		else:

			self._retrieve_messages(self, REQUEST=REQUEST, RESPONSE=RESPONSE)

			RESPONSE.redirect(REQUEST['HTTP_REFERRER'])
	

	def _retrieve_messages(self, hack, REQUEST=None, RESPONSE=None):
		"""Retrieves messages from the local server."""

		message_filters = []

		for filter_folder in self.objectValues('MessageFilterFolder'):

			for message_filter in filter_folder.objectValues('MessageFilter'):

				message_filters.append(message_filter)

		primary_folder = self[self.primary_folder]

		#mbox = mailbox.UnixMailbox(open('%s/Products/%s/test-mailbox' % \
		#				(SOFTWARE_HOME, self.product_name)))

		# FIXME -- poor hack
		# Move to updating of properties?

		checkname = re.compile(r'^([A-Z]|[a-z]|[0-9])*')

		if not checkname.search(self.local_username):

			raise 'NoWay', 'You\'re not getting away with this username.. %s' \
				  % self.local_username

		mbox = mailbox.UnixMailbox(
			open('/var/spool/mail/'+self.local_username))

		#mbox = mailbox.UnixMailbox(
		#	open('/tmp/mailbox'))

		while 1:

			_message = mbox.next()

			if not _message:

				break

			_message.fp.seek(0)
			_message = RFC822Message.util.my_string_io(_message.fp.read())

			message = RFC822Message.rfc822_message_parse.rfc822_message_parse(
				_message)

			for message_filter in message_filters:

				if message_filter.match(message.headers):

					RFC822Message.util.store_message(
						self[message_filter.message_container],
						message, readonly=1)

					break

			else:

				RFC822Message.util.store_message(
					primary_folder, message, readonly=1)


	def gum_enviroment(self):
		"""Returns a truth value to whether the requesting object is within a
		GUM enviroment."""

		# FIXME -- this is stupid

		return 1


	def compose_mail(self, REQUEST=None, RESPONSE=None):
		"""Creates a new instance of a message and redirects to it."""

		subject = ''
		body = ''
		recipients = []
		newsgroups = []

		if REQUEST:

			if REQUEST.has_key('subject') and REQUEST['subject']:

				subject = subject

			if REQUEST.has_key('body') and REQUEST['body']:

				body = body

			if REQUEST.has_key('recipients') and REQUEST['recipients']:

				recipients = REQUEST['recipients']

				if type(recipients) == type(''):

					recipients = [recipients]

			if REQUEST.has_key('newsgroups') and REQUEST['newsgroups']:

				newsgroups = REQUEST['newsgroups']

				if type(newsgroups) == type(''):

					newsgroups = [newsgroups]


		temporary_folder = self.get_temporary_folder()

		new_id = str(self.get_unique_id())

		RFC822Message.rfc822_message.manage_add_rfc822_message(temporary_folder,
								new_id,
								sender=self.email_address,
								body=body,
								subject=subject)

		message_reference = temporary_folder[new_id]

		if recipients:

			for recipient in recipients:

				new_id = str(self.get_unique_id())

				message_reference['to'].manage_add_rfc822_address(
									new_id,
									recipient)

		if newsgroups:

			for newsgroup in newsgroups:

				message_reference.add_newsgroup(newsgroup)


		if RESPONSE:

			RESPONSE.redirect(message_reference.absolute_url())

		else:

			return message_reference


	def get_all_available_newshosts(self):
		"""Returns a list of all available news_host objects."""

		news_hosts = []

		reference = self

		while 1:

			try:

				_news_hosts = reference.objectValues('NewsHost')

			except AttributeError:

				# We've come to the end of the road..

				break

			if _news_hosts:

				news_hosts.extend(_news_hosts)

			reference = reference.getParentNode()

		return news_hosts


default__class_init__(gum)
