###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.3 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import Products
import base_item
import string
import smtplib # FIXME
import news_letter

import category

manage_addNewsFolderForm = HTMLFile('newsfolderAdd', globals())

def manage_addNewsFolder(self, id, title, description, newsletter_mailhost,
						 newsletter_originator,amplifier,
						 REQUEST=None, RESPONSE=None):
	"Add an instance of the NewsFolder class"

	self._setObject(id, NewsFolder(id, title, description, newsletter_mailhost,
								   newsletter_originator, amplifier))

	if RESPONSE and REQUEST:

		RESPONSE.redirect(REQUEST['URL3'])


class NewsFolder(
	#Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'NewsFolder'
	icon = 'misc_/NewsFolder/Folder_icon.gif'
	#default_catalog = 'Catalog'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		{'id':'newsletter_mailhost', 'type':'string'},
		{'id':'newsletter_originator', 'type':'string'},
		{'id':'amplifier', 'type':'string'},
		# What was this thing doing here anyway?
		#{'id':'published_via_email', 'type':'boolean'},
		)

	meta_types = (
		{'name':'Category', 'action':'manage_addCategoryForm'},
		)

	manage_addCategoryForm = category.manage_addCategoryForm
	manage_addCategory = category.manage_addCategory
	categoryAdd2 = category.categoryAdd2

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add Category',
		 ('manage_addCategory', 'manage_addCategoryForm')),
		('View NewsFolders',
		 ('',)),
		)

	index_html = HTMLFile('newsfolderIndex', globals())
	display = HTMLFile('news_folder_display', globals())
	make_newsletter = HTMLFile('make_newsletter', globals())
	newsletter_sent = HTMLFile('newsletter_sent', globals())

	base_edit = HTMLFile('news_folder_edit', globals())

	product_name = 'NewsFolder'


	def __init__(self, id, title='', description='', newsletter_mailhost='',
				 newsletter_originator='', amplifier=''):
		"""Initialize the class instance,  and add default forums."""

		self.id = id
		self.title = title
		self.description = description
		self.newsletter_mailhost = newsletter_mailhost
		self.newsletter_originator = newsletter_originator
		self.amplifier = amplifier
		self.published_via_email = 0

		self.counter = 0

		try:

			self.index_object()

		except AttributeError:

			# ZCatalog isn't available

			pass

		self.manage_addCategory('Internal', 'Internal', 'This category contains internal news', public=0)
		self.manage_addCategory('Public', 'Public', 'This container contains public news', public=1)

		self.manage_addFolder('News_letters', 'Newsletters sent to the outside world..')
		self['News_letters'].manage_addDTMLMethod('index_html', '', HTMLFile('news_letter_folder_index', globals()))

		self.manage_addDTMLMethod('standard_html_header', 'Helpful navigation and such',
								  HTMLFile('standard_html_header', globals()))
		self.manage_addDTMLMethod('standard_html_footer', '',
								  HTMLFile('standard_html_footer', globals()))


	def get_id(self):
		"""Returns a unique identifier."""

		try:

			return self.getParentNode().get_id()

		except:

			self.counter = self.counter + 1
			
			return self.counter


	def sequence_color(self, context=None):
		"""Renders the appropriate color."""

		if not context:

			raise


	def make_newsletter_post(self, REQUEST=None, RESPONSE=None):
		"""Makes a newsletter."""

		con = smtplib.SMTP(self.newsletter_mailhost or 'localhost')

		news_items_body = ''

		try: REQUEST['news_item']
		except AttributeError: raise 'Error', 'No news items selected'

		for news_item in REQUEST['news_item']:

			category, news_item = string.split(news_item, ':')

			news_item = self[category][news_item]

			news_items_body = news_items_body + """%s

%s

<url:%s>
----
""" % (news_item.title, news_item.short_body, news_item.absolute_url())

			news_item.published_via_email = 1

		newsletter = """Subject: Newsletter from %s - %s

%s

%s

%s
""" % ('FIXME', # FIXME
	   REQUEST['title'], REQUEST['prelude'], news_items_body, REQUEST['postlude'])

		con.sendmail(self.newsletter_originator, [self.amplifier], newsletter)

		news_letter.manage_add_news_letter(self['News_letters'], str(self.get_id()),
										   REQUEST['title'], REQUEST['prelude'], REQUEST['postlude'],
										   newsletter)

		RESPONSE.redirect('newsletter_sent')


	def get_unmailed_news(self, number_of_news_items=None):
		"""Returns a list of news items not yet posted via email, if
		the maximum number of news items is specified, it will return the
		oldest items."""

		news_items = self.get_news(number_of_news_items=None)

		news_items = filter(lambda x: not x.published_via_email, news_items)

		#raise str(len(news_items))

		news_items.sort(
			lambda x,y: cmp(
			x.bobobase_modification_time, y.bobobase_modification_time
			))

		if number_of_news_items:

			return news_items[-number_of_news_items:]

		else:

			return news_items

		
	def get_news_sorted(self, number_of_news_items=None):
		"""Returns a list of news items sorted, if the maximum number
		of news items is specified, it will return the most recent items."""

		news_items = self.get_news()

		news_items.sort(
			lambda x,y: cmp(
			x.bobobase_modification_time, y.bobobase_modification_time
			))

		if number_of_news_items:

			return news_items[:number_of_news_items]

		else:

			return news_items


	def get_news(self, number_of_news_items=10):
		"""Returns all news messages."""

		news_items = []

		for category in self.objectValues('Category'):

			map(news_items.append, category.get_news())

		if number_of_news_items:

			return news_items[:number_of_news_items]

		else:

			return news_items


default__class_init__(NewsFolder)
