###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Erik Enge <erik@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.7 $

__doc__ = "Documentation string"
__version__ = "0.1.0"

debug = 1

from Globals import HTMLFile
import string

class base_item:
	"Documentation string"

	meta_types = ()

	base_delete = HTMLFile('base_delete', globals())
	base_edit = HTMLFile('base_edit', globals())

	color_id0 = 'lightblue'
	color_body0 = 'white'
	color_body1 = 'lightgrey'
	color_action1 = 'lightblue'
	color_action0 = color_action1
	color_header0 = 'lightyellow'
	color_header1 = '#6699cc'
	color_header_description0 = '#99cc66'
	color_sequence_even0 = '#cccccc'
	color_sequence_odd0 = '#aeaeae'
	table_cellpadding0 = '1'
	table_cellspacing0 = '1'

	def local_navigation(self, object, REQUEST):
		"Return navigation local to this class."
		
		if len(self.meta_types) != 0:
			block = '<form action="'+str(REQUEST.URL1)+'/" method="GET">\n'
			block = block + '\t<select name=":method">\n'

			for sub_class in self.meta_types:
				block = block + '\t\t<option value="%s">%s</option>\n' % \
						(sub_class['action'], sub_class['name'])
			block = block + '\t</select>\n\t<input type="submit" value=" Add ">\n</form>\n'

		else:
			
			block = ""
			
		block = block + '<p><a href="../base_delete?ids=%s">Delete</a><br>' % object.id
		block = block + '<a href="base_edit">Edit</a></p>' 
		block = block + '<p><a href="/Control_Panel/Products/%s/Help/' % (self.product_name) +	string.lower(object.meta_type)+'.stx">Help</a></p>'

		return block

	def get_properties(self):
		"Return a list of properties for this object."

		return self._properties


	def base_edit_properties(self, REQUEST):
		"Update properties, with a nice redirect."

		if REQUEST.has_key('__ignore_keyerror') and REQUEST['__ignore_keyerror']:

			for property in self._properties:

				try:

					exec("self."+str(property['id'])+" = REQUEST."+str(property['id']))

				except KeyError:

					pass

		else:

			for property in self._properties:

				exec("self."+str(property['id'])+" = REQUEST."+str(property['id']))

		return REQUEST.RESPONSE.redirect(self.absolute_url())


	def navigation(self, object, separator_html=' &gt; '):
		""

		path = []

		physical_path = list(self.getPhysicalPath())
		physical_path.reverse()

		for element_index in range(len(physical_path)):

			path.append("""<a href="%s">%s</a>""" % \
						('../' * element_index,
						 physical_path[element_index] or 'base'))


		path.reverse()

		return string.join(path, separator_html)


	def footer(self, context):
		""

		return ""


	def product_installed(self, product_name):
		"""A method that returns a truth value if given
		product is installed."""

		if product_name in self.Control_Panel.Products.objectIds():

			return 1

		else:

			return 0


	def __call__(self):
		"""Brings up a useful display of the object."""

		return_string = "<class %s at %s" % (self.__class__.__name__,
											 id(self))

		for property in self._properties:

			return_string = return_string + "%s: %s\n" %(property,
													   self[property])

		else:

			return_string = return_string + '\n>'

		return return_string


	def manage_delete(self, ids=[], URL1=None, RESPONSE=None):
		"""Deletes given ids."""

		if type(ids) is type(''):

			ids = [ids]

		self.manage_delObjects(ids)

		if URL1 is not None and RESPONSE is not None:

			RESPONSE.redirect(URL1)


	def redirect_next_id(self, REQUEST, RESPONSE):
		"""Returns the 'next' object id."""

		object_id = REQUEST['__object_id']
		
		if REQUEST.has_key('__meta_type') and REQUEST['__meta_type']:

			meta_type = REQUEST['__meta_type']

		else:

			meta_type = None

		if REQUEST.has_key('__resolve_url') and REQUEST['__resolve_url']:

			reference = REQUEST.resolve_url(REQUEST['__resolve_url'])

		else:

			reference = self

		objectIds = reference.objectIds(spec=meta_type)

		try:

			next = objectIds[objectIds.index(object_id) + 1]

		except IndexError:

			# We've got the last id, wrapping around to the first one.

			next = objectIds[0]

		if debug: print next

		RESPONSE.redirect(reference.absolute_url() + '/' + next)


	def redirect_previous_id(self, REQUEST, RESPONSE):
		"""Returns the 'previous' object id."""

		object_id = REQUEST['__object_id']

		if REQUEST.has_key('__meta_type') and REQUEST['__meta_type']:

			meta_type = REQUEST['__meta_type']

		else:

			meta_type = None

		if REQUEST.has_key('__resolve_url') and REQUEST['__resolve_url']:

			reference = REQUEST.resolve_url(REQUEST['__resolve_url'])

		else:

			reference = self

		objectIds = reference.objectIds(spec=meta_type)

		try:

			previous = objectIds[objectIds.index(object_id) - 1]

		except ValueError:

			# We've got the first id in the list, wrap it.

			previous = objectIds[-1]

		if debug: print REQUEST['__object_id'], \
				REQUEST['__resolve_url']

		if debug: print previous

		RESPONSE.redirect(reference.absolute_url() + '/' + previous)


	def manage_delete_all(self, REQUEST=None, RESPONSE=None):
		"""Method that deletes all messages in folder."""

		if REQUEST is not None:

			if REQUEST.has_key('confirm') and \
				REQUEST['confirm']:

				pass

			else:

				RESPONSE.redirect(self.absolute_url())
				return

		self.manage_delete(ids=self.objectIds())

		RESPONSE.redirect(self.absolute_url())
