###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.7 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import rfc822_attachment
import rfc822_address_container
import rfc822_address
import base_item
import cStringIO
import mimetools
import mimify
import extended_mime_writer
import string

add_rfc822_message_redirect = HTMLFile('rfc822_message_add_redirect',
					globals())
manage_add_rfc822_message_form = HTMLFile('rfc822_message_add', globals())


def manage_add_rfc822_message(self, id, sender='', reply_to='', body='',
				subject='', message_id='', REQUEST=None):
    "Add an instance of the rfc822_message class"

    self._setObject(id,
			rfc822_message(id, sender, body, subject,
				message_id=message_id, reply_to=reply_to))

    if REQUEST is not None:

        return self.manage_main(self, REQUEST)


class rfc822_message(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'RFC822Message'
	icon = 'misc_/RFC822Message/Folder_icon.gif'

	_properties = (
		{'id':'sender', 'type':'string'},
		{'id':'reply_to', 'type':'string'},
		{'id':'body', 'type':'text'},
		{'id':'readonly', 'type':'boolean'},
		{'id':'read', 'type':'boolean'},
		)

	meta_types = (
		{'name':'RFC822Attachment',
		 'action':'manage_add_rfc822_attachment_form'},
		{'name':'RFC822Address',
		 'action':'manage_add_rfc822_address_form'},
		)

	manage_add_rfc822_attachment_form = rfc822_attachment.manage_add_rfc822_attachment_form
	manage_add_rfc822_attachment = rfc822_attachment.manage_add_rfc822_attachment
	rfc822_attachment_add_redirect = rfc822_attachment.rfc822_attachment_add_redirect

	manage_add_rfc822_address_container_form = rfc822_address_container.manage_add_rfc822_address_container_form
	manage_add_rfc822_address_container = rfc822_address_container.manage_add_rfc822_address_container
	rfc822_address_container_add_redirect = rfc822_address_container.rfc822_address_container_add_redirect


	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add RFC822Attachment',
		 ('manage_add_rfc822_attachment', 'manage_add_rfc822_attachment_form')),
		('Add RFC822Address',
		 ('manage_add_rfc822_address', 'manage_add_rfc822_address_form')),
		('View RFC822Messages',
		 ('',)),
		)

	index_html = HTMLFile('rfc822_message_index', globals())

	view_index_html = HTMLFile('rfc822_message_view_index', globals())
	edit_index_html = HTMLFile('rfc822_message_edit_index', globals())


	def mark_as_read(self):
		"""Marks the message as read."""

		self.read = 1


	def copy(self, temporary_folder=None):
		"""Copies message to temporary folder."""

		if temporary_folder is not None:

			temporary = temporary_folder

		else:

			temporary = self.get_temporary_folder()

		message_copy = self.getParentNode().manage_copyObjects(self.id)
		temporary.manage_pasteObjects(cb_copy_data=message_copy)

		# We do this so we never get id conflicts.

		new_id = str(self.get_unique_id())

		temporary.manage_renameObject(self.id, new_id)

		message_new = temporary[new_id]

		return message_new


	def delete_address_fields(self):
		"""Removes all recipients."""

		for address_field in ['to', 'cc', 'bcc']:

			for address in self[address_field].objectValues():

				self[address_field].manage_delObjects(address.id)


	def forward(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for forwarding."""

		if REQUEST.has_key('temporary_folder'):

			temporary_folder = REQUEST.resolve_url(REQUEST['temporary_folder'])

		else:

			temporary_folder = None

		message_new = self.copy(temporary_folder=temporary_folder)

		message_new.subject = self.render_forward_subject()
		message_new.body = self.render_forward_body()

		message_new.delete_address_fields()

		message_new.readonly = 0

		RESPONSE.redirect(message_new.absolute_url())



	def reply(self, REQUEST=None, RESPONSE=None, all=None):
		"""Method that prepares a message for replying."""

		new_id = str(self.get_unique_id())

		new_body = '> ' + string.join(string.split(self.body,'\n'), '\n> ')

		if self.headers.has_key('references'):

			references = string.split(self.headers['references'])

		else:

			references = []

		headers = {}

		try:

			references.append(self.headers['message-id'])

		except KeyError:

			# There was no message id..

			pass

		if references:

			headers['references'] = string.join(references, ' ')

		if REQUEST is not None and REQUEST.has_key('temporary_folder'):

			# FIXME

			temporary_folder = REQUEST.resolve_url(
						REQUEST['temporary_folder'])

		else:

			temporary_folder = self.get_temporary_folder()

		manage_add_rfc822_message(temporary_folder, new_id,
					  sender=temporary_folder.email_address,
					  body=new_body,
					  subject=self.render_reply_subject())

		temporary_folder[new_id]['to'].manage_add_rfc822_address(
			str(self.get_unique_id()), self.sender, comment='', realname='')

		temporary_folder[new_id].references = references[:]

		if all is not None:

			for address_field in self.get_filled_address_fields():
				
				for rfc822_address in self[address_field].objectValues(
					'RFC822Address'):

					temporary_folder[new_id]['cc'].manage_add_rfc822_address(
						str(self.get_unique_id()), rfc822_address.address,
						comment=rfc822_address.comment,
						realname=rfc822_address.realname)

			temporary_folder[new_id].newsgroups = self.newsgroups[:]

		RESPONSE.redirect(temporary_folder[new_id].absolute_url())


	def reply_all(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for replying."""

		self.reply(REQUEST=REQUEST, RESPONSE=RESPONSE, all=1)


	def render_reply_subject(self):
		"""Method for rendering an appropriate subject upon replies."""

		subject = self.headers['subject'] or self.subject

		if string.lower(subject)[:3] == 're:':

			return subject

		else:

			return 'Re: ' + subject


	def render_forward_subject(self):
		"""Method for rendering an appropriate subject upon forwards."""

		subject = self.subject or self.headers['subject']

		if string.lower(subject)[:4] == 'fwd:':

			return subject

		else:

			return 'Fwd: ' + subject


	def render_forward_body(self):
		"""Quotes the body of the forwarded message."""

		new_body = string.split(self.body, '\n')

		new_body = '>> ' + string.join(new_body, '\n>> ')

		return new_body
									   

	def get_filled_address_fields(self):
		"""Returns address fields that contain one or more addresses."""

		address_fields = []

		for address_field in ['to', 'cc', 'bcc']:

			if len(self[address_field].objectIds('RFC822Address')):

				address_fields.append(address_field)

		return address_fields


	def render_sender(self):
		"""Renders the 'from' address field."""

		return self.sender


	def render_newsgroups(self):
		"""Renders the newsgroups header field."""

		return string.join(self.newsgroups, ', ')


	def render_address_field(self, field='to'):
		"""Renders the addresses contained in the given field."""

		return_string = ""
		
		for object in self[field].objectValues():

			return_string = return_string + '%s, ' % object.address

		else:

			return_string = return_string[:-2]

		return return_string


	def update(self, REQUEST=None, RESPONSE=None):
		"""Updates message data."""

		self.body = REQUEST['body']
		self.subject = REQUEST['subject']
		newsgroups = map(string.strip, string.split(REQUEST['newsgroups'], ','))
		self.newsgroups = filter(None, newsgroups)

		if RESPONSE is not None:

			RESPONSE.redirect(self.absolute_url())


	def save_draft(self, REQUEST=None, RESPONSE=None):
		"""Saves a draft of this message."""

		self.update(REQUEST=REQUEST)

		draft_folder = self.get_draft_folder()

		copy_data = self.getParentNode().manage_copyObjects(self.id)
		draft_folder.manage_pasteObjects(copy_data)

		return 'draft saved..'


	def raw_body(self, RESPONSE):
		"""Returns the original message body."""

		RESPONSE.setHeader('content-type', 'text/plain')

		return self._raw_body


	def send(self, REQUEST=None, RESPONSE=None):
		"""Sends the message."""

		if REQUEST is not None:

			self.update(REQUEST=REQUEST)

		# Do message format magic and such calls here

		if not self.get_filled_address_fields() and not self.newsgroups:

			raise ValueError, 'No recipients or newsgroups specified'

		self.headers['message_id'] = str(self.get_unique_id()) + '.' + \
						mimetools.choose_boundary() + '@' + \
						string.split(self.email_address, '@')[-1]
									   

		if self.get_filled_address_fields():

			mail_gateway = self.get_mail_gateway()

			res = mail_gateway.post(self)

		if filter(None, self.newsgroups):

			news_host = self.get_news_host()
			
			news_host.post(self)


		if RESPONSE:

			# What screen do we return to here?
			RESPONSE.setHeader('content-type', 'text/plain')
			return self.raw_message()
				   


	def add_recipient(address_field='to', address='', realname='', comment=''):
		"""Adds a recipient to the given address field."""

		new_id = str(self.get_unique_id())

		self[address_field].manage_add_rfc822_address(new_id, address,
							comment=comment,
							realname=realname)


	def set_header(self, header_name, header_value):
		"""Sets a mail message header."""

		self.headers[header_name] = header_value


	def add_newsgroup(self, newsgroup):
		"""Adds a newsgroup to the message."""

		try:

			self.newsgroups.index(newsgroup)
			return

		except ValueError:

			self.newsgroups.append(newsgroup)


	def get_recipients(self):
		"""Returns a list of addresses."""

		recipient_list = []

		for address_field in self.get_filled_address_fields():

			for rfc822_address in self[address_field].objectValues(
				'RFC822Address'):

				recipient_list.append(rfc822_address.address)

		return recipient_list


	def get_attachments(self):
		"""Returns a list of RFC822Attachment objects."""

		return self.objectValues('RFC822Attachment')


	def raw_message(self):
		"""Renders an RFC822 compliant message."""

		rendered_message = cStringIO.StringIO()

		writer = extended_mime_writer.extended_mime_writer(rendered_message)

		writer.addheader('From', self.sender)

		for header_key in self.headers.keys():

			writer.addheader(header_key, self.headers[header_key])

		for address_field in self.get_filled_address_fields():

			if address_field != 'bcc':
			
				writer.addheader(
					string.capitalize(address_field),
					mimify.mime_encode_header(
						self.render_address_field(
							field=address_field)))

		if self.newsgroups:

			writer.addheader('Newsgroups', self.render_newsgroups())

		if self.headers['message_id']:

			writer.addheader('Message-ID', '<' + self.message_id + '>')

		else:

			raise 'MessageIDException', 'No message-id specified'

		if self.references:

			writer.addheader('References', string.join(self.references, ' '))
		
		if self.get_attachments():

			writer.addheader('Mime-Version', '1.0')

		writer.addheader('Date', self.ZopeTime().rfc822())

		writer.addheader('X-Mailer', '%s (%s), %s' % \
			('Zope GUM', self.get_gum_version(), self.get_gum_website()))

		writer.flushheaders()

		if self.get_attachments():

			writer.startmultipartbody('mixed')

			mime_part = writer.nextpart()

			mime_part_writer = mime_part.startbody('text/plain')

			mimetools.encode(cStringIO.StringIO(self.body),
						 mime_part_writer, 'quoted-printable')

			for attachment in self.get_attachments():

				mime_part = writer.nextpart()

				tmp_headers = {'content-transfer-encoding':
						attachment.encoding or 'base64'}
				tmp_headers.update(attachment.headers)

				mime_part_writer.startbody(
					attachment.content_type(),
					[('filename', attachment.filename)],
					tmp_headers)

				mime_part_writer(attachment.encode())

			else:

				writer.lastpart()

		else:

			# We have to do this to add headers..

			for header_name in self.headers.keys():

				writer._fp.write('%s: %s\n' % (header_name, self.headers[header_name]))

			writer._fp.write('Content-Transfer-Encoding: quoted-printable\n')
			writer._fp.write('Subject: %s\n' % self.subject)

			body_writer = writer.startbody('text/plain', [], # FIXME
						{'Content-Transfer-Encoding':
						'quoted-printable'})

			body = cStringIO.StringIO(self.body)
			body.seek(0)

			mimetools.encode(body, body_writer, 'quoted-printable')

		if self.headers.has_key('copy-to-folder'):

			# FIXME

			pass

		return rendered_message.getvalue()


	def get_newsgroups(self):
		"""Returns a list of newsgroups."""

		return self.newsgroups


	def __init__(self, id, sender, body, subject, message_id='', reply_to=''):
		"Initialize the class instance"

		self.id = id

		self.sender = sender
		self.reply_to = reply_to
		self.body = body
		self.subject = subject
		self.message_id = message_id

		self.read = 0

		self.newsgroups = []
		self.headers = {}

		# FIXME?
		self.references = []

		self.manage_add_rfc822_address_container('to', title='to')
		self.manage_add_rfc822_address_container('cc', title='cc')
		self.manage_add_rfc822_address_container('bcc', title='bcc')

		try:

			self.index_object()

		except AttributeError:

			# No ZCatalog instance available..

			pass


default__class_init__(rfc822_message)


